/*
    Copyright (C) 2005 - 2007 Mikael Hgdahl - dronten@gmail.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser BaseSetupPanel Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser BaseSetupPanel Public License for more details.

    You should have received a copy of the GNU Lesser BaseSetupPanel Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

package com.googlepages.dronten.jripper.music;

import com.googlepages.dronten.jripper.Constants;
import com.googlepages.dronten.util.Log;
import com.googlepages.dronten.util.Pref;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * CDDB query by http protocoll.
 */
public class CDDB {
    private static final String CGI_READ = "/~cddb/cddb.cgi?cmd=cddb+read+";
    private static final String CGI_QUERY = "/~cddb/cddb.cgi?cmd=cddb+query+";
    private static final String THE_END1 = "&hello=";
    private static final String THE_END2 = "+localhost+jRipper+2007.01&proto=6";

    private static final Pattern FOUND_EXACT_MATCH = Pattern.compile("^(200)\\s(\\S*)\\s(\\S*)\\s(.*)");
    private static final Pattern FOUND_EXACT_MATCHES = Pattern.compile("^(210).*");
    private static final Pattern FOUND_INEXACT_MATCHES = Pattern.compile("^(211).*");
    private static final Pattern FOUND_MATCHES = Pattern.compile("(\\S*)\\s(\\S*)\\s(.*)");
    private static final Pattern FOUND_NO_MATCH = Pattern.compile("(202) No.*");

    private static final Pattern RECORD_DTITLE = Pattern.compile("DTITLE=(.*)");
    private static final Pattern RECORD_DYEAR = Pattern.compile("DYEAR=(.*)");
    private static final Pattern RECORD_DGENRE = Pattern.compile("DGENRE=(.*)");
    private static final Pattern RECORD_TTITLEN = Pattern.compile("TTITLE(\\d+)=(.*)");
    private static final Pattern RECORD_EXTD = Pattern.compile("EXTD=(.*)");

    public static int CDDB_MATCH = 0;


    /**
     * Read album data from server.
     * A query must be done first before getting the actual data.
     * http://uk.freedb.org/~cddb/cddb.cgi?cmd=cddb+read+folk+440a5905&hello=joe+my.host.com+appname+0.1&proto=6
     *
     * @param log
     * @param server
     * @param port
     * @param album
     * @param discID - This info comes from the query call
     * @return true if ok
     */
    public static boolean getAlbumData(Log log, String server, int port, Album album, DiscID discID) {
        String readCmd = "";
        String reply = "";
        BufferedReader inputStream = null;
        URL url = null;
        URLConnection con = null;
        int trackCount = -1;
        String lastTitle = "-1";

        album.aArtist = "";
        album.aAlbum = "";
        album.aYear = "";
        album.aComment = "";

        try {
            readCmd = CGI_READ + discID.aGenre + "+" + discID.aDiscID;
            readCmd += THE_END1;
            readCmd += Pref.getPref(Constants.CDDB_EMAIL_KEY, Constants.CDDB_EMAIL_DEFAULT);
            readCmd += THE_END2;
            log.addTime(1, server + ":" + port + "/" + readCmd);

            url = new URL("http", server, port, readCmd);
            con = url.openConnection();
            inputStream = new BufferedReader(new InputStreamReader(con.getInputStream(), "UTF-8"));
            reply = inputStream.readLine();

            while (reply != null) {
                log.addTime(1, reply);

                Matcher m1 = RECORD_DTITLE.matcher(reply);
                Matcher m2 = RECORD_DYEAR.matcher(reply);
                Matcher m3 = RECORD_DGENRE.matcher(reply);
                Matcher m4 = RECORD_TTITLEN.matcher(reply);
                Matcher m5 = RECORD_EXTD.matcher(reply);

                if (m1.find()) {
                    album.aArtist += m1.group(1);
                }
                else if (m2.find()) {
                    album.aYear = m2.group(1);
                }
                else if (m3.find()) {
                    album.aGenre = m3.group(1);
                }
                else if (m4.find()) {
                    if (trackCount < album.aTracks.size()) {

                        if (m4.group(1).equalsIgnoreCase(lastTitle)) {
                            // Append title string
                            Track track = album.aTracks.get(trackCount);
                            track.setName(track.aName + m4.group(2));
                        }
                        else {
                            trackCount++;
                            album.aTracks.get(trackCount).setName(m4.group(2));
                        }
                        lastTitle = m4.group(1);
                    }
                }
                else if (m5.find()) {
                    album.aComment += m5.group(1);
                }

                reply = inputStream.readLine();
            }
            String[] tmp = album.aArtist.split(" / ", 2);
            if (tmp.length > 0) {
                album.aArtist = tmp[0];
            }
            if (tmp.length > 1) {
                album.aAlbum = tmp[1];
            }

            album.aComment = album.aComment.replaceAll("\\\\n", ". ");
            album.aDiscID = discID.aDiscID;

            album.aArtist = album.aArtist.trim();
            album.aAlbum = album.aAlbum.trim();
            album.aYear = album.aYear.trim();
            album.aComment = album.aComment.trim();
            for (Track track : album.aTracks) {
                track.aName = track.aName.trim();
            }

            inputStream.close();
            if (album.aTracks.size() > 0) {
                CDDB.CDDB_MATCH = 2;
            }
            return true;
        }
        catch (Exception e) {
            log.addTime(1, e.getMessage());
            return false;
        }
    }


    /**
     * Query a cddb server if a record exists.
     * http://uk.freedb.org/~music/music.cgi?cmd=music+query+0xf12321+18+0+18020+33590+53442+69152+79436+97579+114228+137805+151740+164875+180579+196818+214996+229914+246361+274717+306266+4417&hello=joe+my.host.com+testing+0.1&proto=6
     *
     * @param log
     * @param server
     * @param port
     * @param album
     * @return All found records or an empty vector if none is found
     */
    public static Vector<DiscID> queryServerForAlbum(Log log, String server, int port, Album album) {
        String queryCmd = "";
        String reply = "";
        BufferedReader inputStream = null;
        URL url = null;
        URLConnection con = null;
        Vector<DiscID> availableAlbums = new Vector<DiscID>();
        boolean ok = false;

        CDDB_MATCH = 0;

        try {
            queryCmd = CGI_QUERY + album.aDiscID + "+" + album.aTracksString;
            for (Track t : album.aTracks) {
                queryCmd += "+" + t.aSector;
            }
            queryCmd += "+" + album.aSeconds;
            queryCmd += THE_END1;
            queryCmd += Pref.getPref(Constants.CDDB_EMAIL_KEY, Constants.CDDB_EMAIL_DEFAULT);
            queryCmd += THE_END2;
            log.addTime(1, server + ":" + port + queryCmd);

            url = new URL("http", server, port, queryCmd);
            con = url.openConnection();
            inputStream = new BufferedReader(new InputStreamReader(con.getInputStream(), "UTF-8"));
            reply = inputStream.readLine();

            while (reply != null) {
                log.addTime(1, reply);

                Matcher m1 = FOUND_EXACT_MATCH.matcher(reply);
                Matcher m2 = FOUND_EXACT_MATCHES.matcher(reply);
                Matcher m3 = FOUND_INEXACT_MATCHES.matcher(reply);
                Matcher m4 = FOUND_MATCHES.matcher(reply);
                Matcher m5 = FOUND_NO_MATCH.matcher(reply);

                if (m5.find()) {
                    break;
                }
                else if (m1.find()) {
                    availableAlbums.add(new DiscID(m1.group(2), m1.group(3), m1.group(4)));
                    break;
                }
                else if (m2.find() || m3.find()) {
                    // Eat this line
                    ok = true;
                }
                else {
                    if (!ok) {
                        break;
                    }
                    else if (m4.find()) {
                        availableAlbums.add(new DiscID(m4.group(1), m4.group(2), m4.group(3)));
                    }
                }

                reply = inputStream.readLine();
            }
            inputStream.close();
            if (availableAlbums.size() > 0) {
                CDDB_MATCH = 1;
            }
        }
        catch (Exception e) {
            log.addTime(1, e.getMessage());
            availableAlbums.clear();
        }

        return availableAlbums;
    }
}
